@@ -15,15 +15,16 @@ or::
     env = Environment()
     PYTOOL()(env)
 
-This does two things:
+This does three things:
 
-1. Installs a new builder: PyScript.
-2. Installs a new scanner for python scripts.
+1. Installs a builder: PyScript.
+2. Installs a builder: PyScons.
+3. Installs a new scanner for python scripts.
 
 PyScript
 --------
 
-This Builder has a few additional abilities. 
+This Builder runs python scripts and modules. 
 
 First, it will automatically find the ".py" files referred to when 
 running a module as a script with the '-m' option. For example
@@ -47,6 +48,46 @@ Just like Command, multiple steps can be used to create a file::
    env.PyScript("out", ["step1.py", "step2.py"], 
         ["python ${SOURCES[0]} > temp", "python ${SOURCES[1]} > $TARGET", Delete("temp")])
 
+PyScons (experimental)
+-------
+
+This Builder enables running a python script as if it is a scons script. 
+
+This is distinct from SConscript which functions like an include. Instead, PyScons spawns a new scons process.
+Spawning a new process allows for temporary files to be automatically deleted without triggering a rebuild.
+
+To use this builder, create a .py file with, for example, the following code in a file (my_scons.py)::
+
+    from pyscons import PySconsSetup
+    targets, sources, basename = PySconsSetup()
+   
+    temp = basename + ".temp"
+
+    PyScript(temp, ["step1.py"] + sources, capture="out")
+    Pyscript(targets, ["step2.py", temp], capture="out")
+ 
+Now, this file can be called from a SConstruct file like so::
+
+    PyScons(targets, sources, "my_scons.py", options = "-j4")
+
+The temp file be generated if it is required to generate targets, but will be immediately deleted.
+This is useful for builders which generate large intermediate files which would should be deleted
+without triggering a rebuild. This can be better than passing a list to the Command function for a 
+few special cases:
+
+1. PyScons enables parallel execution of a multistep submodule(if you pass the -j option to the spawned scons) 
+2. PyScons creates a workflow environment (like Pipeline Pilot) in scons which enables complex tasks to be packaged in scons files for
+use in other scons files.
+3. PyScons can turn intermediate file deletion on and off with a single flag::
+
+    PyScons(targets, sources, "my_scons.py", clean = True) # intermediate file deleted
+    PyScons(targets, sources, "my_scons.py", clean = False) # intermediate file retained
+
+Unfortunately, dependency tracking does not propagate up from the spawned scons. So 
+sources that are not explicitely included in the "sources" variable will not be tracked. There 
+may be no good way around this, short of creating a new scanner and file extension (which I 
+may do if there interest in this feature).
+
 PyScanner
 ---------
 
@@ -87,6 +128,7 @@ from SCons import Util
 from SCons.Scanner import Scanner
 from SCons.Script.SConscript import DefaultEnvironmentCall
 from modulefinder import ModuleFinder as MF
+from SCons.Script import ARGUMENTS, DefaultEnvironment
 
 class PyScanner(MF):
     """
@@ -142,6 +184,21 @@ def _PyScript(env, target, source, command = "python $SOURCES",
     command = [ env.subst(c, target=target, source=subst) for c in command ]
     return env.Command(target, source, command, *args, **kwargs)  
 
+def _PyScons(env, target, source, scons, options="-Q", clean = True):
+    target = ToList(target) 
+    source = ToList(source) 
+
+    args = " ".join(["in%g=%s" % (n+1, str(s)) for n,s in enumerate(source)])
+    args += " " + " ".join(["out%g=%s" % (n+1, str(s)) for n,s in enumerate(target)])
+
+    if env.GetOption("clean") and not clean: env.Execute("scons %s -f%s %s --clean" % (options, scons, args))
+
+    cmd = "scons %s -f${SOURCES[-1]} %s" % (options, args)
+    if clean: cmd = [cmd, "scons %s -f${SOURCES[-1]} %s --clean" % (options, args)]
+
+    return env.Command(target, source + [scons], cmd)
+
+PyScons = DefaultEnvironmentCall("PyScons")
 PyScript = DefaultEnvironmentCall("PyScript")
 
 class PYTOOL:
@@ -156,4 +213,36 @@ class PYTOOL:
         env.Append(SCANNERS = Scanner(function = PyScanner(self.path_filter), 
                     name = "PyScanner", skeys = ['.py', '.pyo', '.pyc'], recursive=self.recursive))
         env.AddMethod(_PyScript, "PyScript")
+        env.AddMethod(_PyScons, "PyScons")
         return env
+
+def PySconsSetup(args=ARGUMENTS, tools=['default', PYTOOL()]):
+    env = DefaultEnvironment(ENV=os.environ, tools=tools)
+
+    sources = []
+    targets = []
+    for a in args:
+        if a[:2] == "in":
+            try:
+                i = int(a[2:])
+                sources.append((i, args[a]))
+            except: pass
+        if a[:3] == "out":
+            try:
+                i = int(a[3:])
+                targets.append((i, args[a]))
+            except: pass
+    sources = [a for i,a in sorted(sources)]
+    targets = [a for i,a in sorted(targets)]
+
+    env.NoClean(targets)
+    
+    try: basename = args["base"] 
+    except: basename = None
+
+    if basename == None: 
+        try: basename = targets[-1]
+        except: basename = "TEMP"
+
+    return targets, sources, basename
+
